==> hlog-0.20-rc1.tar.gz: hlog-0.20-rc1/hlog/contest/ui-work.c <==
/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <ctype.h>

#include <hlog/index.h>

#include "../common/recent.h"
#include "../common/status.h"
#include "contest.h"
#include "script.h"

struct xform work; /* current QSO contest exchange */

static void wrapped_set_field_buffer(struct xform *form, const char *key,
				     const char *value)
{
	size_t i;

	for (i = 0; i < form->nfields; i++) {
		if (strcmp(str_cstr(xform_field_get_private(form, i)), key))
			continue;

		xform_field_set(form, i, value);
		return;
	}
}

static void field_changed(lua_State *L, struct qso *qso)
{
	struct xform *form = &work; /* for simplicity */
	char error[128];
	int ret;

	/* The first change to the form needs to stash the start time */
	if (!qso->start.have)
		ASSERT0(qso_set_datetime(qso, true, NULL));

	ret = script_event_field_change(L, qso, form,
					error, sizeof(error),
					wrapped_set_field_buffer);
	if (ret)
		status_display_error("%s", error);
}

static void abort_qso(lua_State *L, struct qso **_qso)
{
	struct xform *form = &work; /* for simplicity */
	char error[128];
	size_t i;
	int ret;

	/* clear the work form if all went well */
	for (i = 0; i < work.nfields; i++)
		xform_field_clear(form, i);

	/* jump to the first field of both forms */
	xform_driver(form, XFORM_REQ_FIRST_FIELD);

	/* initialize the qso */
	ret = init_contact(L, _qso, error, sizeof(error));
	if (ret)
		status_display_error("%s", error);
}

static void end_qso(lua_State *L, struct qso **_qso)
{
	struct xform *form = &work; /* for simplicity */
	struct qso *qso = *_qso;
	char error[128];
	size_t i;
	int ret;

	ret = script_event_qso_done(L, qso, error, sizeof(error));
	if (ret)
		goto err;

	/*
	 * make sure we have a start/end times - use current time if missing
	 *
	 * If everything went well, we should already have a start time
	 * since the first attempt to type something into the form will
	 * store the timestamp.  If the user didn't input anything and just
	 * used the enter key, we could end up without a start time.
	 */
	if (!qso->start.have)
		ASSERT0(qso_set_datetime(qso, true, NULL));
	if (!qso->end.have)
		ASSERT0(qso_set_datetime(qso, false, NULL));

	/* TODO: check that we have a frequency */

	ret = qso_save(qso, AT_FDCWD);
	if (ret) {
		snprintf(error, sizeof(error), "Failed to save QSO: %s",
			 xstrerror(ret));
		goto err;
	}

	ret = index_save(qso);
	if (ret)
		status_display_error("Warning: Failed to add QSO to index, "
				     "UI may be out-of-sync: %s",
				     xstrerror(ret));

	/* clear the work form if all went well */
	for (i = 0; i < work.nfields; i++)
		xform_field_clear(form, i);

	/* jump to the first field of both forms */
	xform_driver(form, XFORM_REQ_FIRST_FIELD);

	/* start the next qso */
	ret = init_contact(L, _qso, error, sizeof(error));
	if (ret)
		status_display_error("%s", error);

	return;

err:
	status_display_error("%s", error);
}

void ui_work_dispatch(lua_State *L, int ch, struct qso **qso)
{
	struct xform *form = &work; /* for simplicity */

	if (ui_dispatch_generic(form, ch))
		return;

	switch (ch) {
		/* delete & backspace */
		/* FIXME: what's the proper check? 127 works */
		case KEY_BACKSPACE:
		case '\b':
		case 127:
			xform_driver(form, XFORM_REQ_DEL_PREV);
			field_changed(L, *qso); /* inform script */
			break;

		case KEY_DC:
			xform_driver(form, XFORM_REQ_DEL_THIS);
			field_changed(L, *qso); /* inform script */
			break;

		/* navigate between forms */
		case KEY_ENTER:
		case '\n':
			end_qso(L, qso);
			recent_invalidate();
			break;

		case '\x1b': /* escape */
			abort_qso(L, qso);
			break;

		default:
			if (!iscntrl(ch)) {
				xform_driver(form, ch);
				field_changed(L, *qso); /* inform script */
			} else {
				beep();
			}
			break;
	}
}

==> hlog-0.21-rc1.tar.gz: hlog-0.21-rc1/hlog/contest/ui-work.c <==
/*
 * Copyright (c) 2021-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <ctype.h>

#include <hlog/index.h>

#include "../common/recent.h"
#include "../common/status.h"
#include "contest.h"
#include "script.h"

struct xform work; /* current QSO contest exchange */

static void wrapped_set_field_buffer(struct xform *form, const char *key,
				     const char *value)
{
	size_t i;

	for (i = 0; i < form->nfields; i++) {
		if (strcmp(str_cstr(xform_field_get_private(form, i)), key))
			continue;

		xform_field_set(form, i, value);
		return;
	}
}

static void wrapped_set_annunciator_color(struct xform *form, const char *key,
					  const char *value)
{
	size_t i;

	for (i = 0; i < form->nannunc; i++) {
		struct str *label = xform_annunciator_get_label(form, i);

		if (strcmp(str_cstr(label), key)) {
			str_putref(label);
			continue;
		}

		str_putref(label);
		xform_annunciator_set(form, i,
				      xform_color_from_name(value));
		return;
	}
}

static void field_changed(lua_State *L, struct qso *qso)
{
	struct xform *form = &work; /* for simplicity */
	char error[128];
	int ret;

	/* The first change to the form needs to stash the start time */
	if (!qso->start.have)
		ASSERT0(qso_set_datetime(qso, true, NULL));

	ret = script_event_field_change(L, qso, form,
					error, sizeof(error),
					wrapped_set_field_buffer,
					wrapped_set_annunciator_color);
	if (ret)
		status_display_error("%s", error);
}

static void abort_qso(lua_State *L, struct qso **_qso)
{
	struct xform *form = &work; /* for simplicity */
	char error[128];
	size_t i;
	int ret;

	/* clear the work form if all went well */
	for (i = 0; i < work.nfields; i++)
		xform_field_clear(form, i);
	for (i = 0; i < work.nannunc; i++)
		xform_annunciator_clear(form, i);

	/* jump to the first field of both forms */
	xform_driver(form, XFORM_REQ_FIRST_FIELD);

	/* initialize the qso */
	ret = init_contact(L, _qso, error, sizeof(error));
	if (ret)
		status_display_error("%s", error);
}

static void end_qso(lua_State *L, struct qso **_qso)
{
	struct xform *form = &work; /* for simplicity */
	struct qso *qso = *_qso;
	char error[128];
	size_t i;
	int ret;

	ret = script_event_qso_done(L, qso, error, sizeof(error));
	if (ret)
		goto err;

	/*
	 * make sure we have a start/end times - use current time if missing
	 *
	 * If everything went well, we should already have a start time
	 * since the first attempt to type something into the form will
	 * store the timestamp.  If the user didn't input anything and just
	 * used the enter key, we could end up without a start time.
	 */
	if (!qso->start.have)
		ASSERT0(qso_set_datetime(qso, true, NULL));
	if (!qso->end.have)
		ASSERT0(qso_set_datetime(qso, false, NULL));

	/* TODO: check that we have a frequency */

	ret = qso_save(qso, AT_FDCWD);
	if (ret) {
		snprintf(error, sizeof(error), "Failed to save QSO: %s",
			 xstrerror(ret));
		goto err;
	}

	ret = index_save(qso);
	if (ret)
		status_display_error("Warning: Failed to add QSO to index, "
				     "UI may be out-of-sync: %s",
				     xstrerror(ret));

	/* clear the work form if all went well */
	for (i = 0; i < work.nfields; i++)
		xform_field_clear(form, i);
	for (i = 0; i < work.nannunc; i++)
		xform_annunciator_clear(form, i);

	/* jump to the first field of both forms */
	xform_driver(form, XFORM_REQ_FIRST_FIELD);

	/* start the next qso */
	ret = init_contact(L, _qso, error, sizeof(error));
	if (ret)
		status_display_error("%s", error);

	return;

err:
	status_display_error("%s", error);
}

void ui_work_dispatch(lua_State *L, int ch, struct qso **qso)
{
	struct xform *form = &work; /* for simplicity */

	if (ui_dispatch_generic(form, ch))
		return;

	switch (ch) {
		/* delete & backspace */
		/* FIXME: what's the proper check? 127 works */
		case KEY_BACKSPACE:
		case '\b':
		case 127:
			xform_driver(form, XFORM_REQ_DEL_PREV);
			field_changed(L, *qso); /* inform script */
			break;

		case KEY_DC:
			xform_driver(form, XFORM_REQ_DEL_THIS);
			field_changed(L, *qso); /* inform script */
			break;

		/* navigate between forms */
		case KEY_ENTER:
		case '\n':
			end_qso(L, qso);
			recent_invalidate();
			break;

		case '\x1b': /* escape */
			abort_qso(L, qso);
			break;

		default:
			if (!iscntrl(ch)) {
				xform_driver(form, ch);
				field_changed(L, *qso); /* inform script */
			} else {
				beep();
			}
			break;
	}
}
